import datetime
import os

import requests
from flask import Flask, render_template, request, abort
from flask_bootstrap import Bootstrap
from flask_caching import Cache
from flask_migrate import Migrate
from flask_nav import Nav
from flask_nav.elements import Navbar, Text, View
from flask_sqlalchemy import SQLAlchemy

from ui import gitlab, config, models

app = Flask(__name__)
Bootstrap(app)
app.config.from_object(config)
cache = Cache(app)
models.db.init_app(app)
migrate = Migrate(app, models.db)
nav = Nav(app)

nav.register_element('top', Navbar(
    "LineageOS Builds",
    View('Builds', '.index'),
    View('Runners', '.runners')
))

headers = {'Private-Token': os.environ.get('GITLAB_TOKEN', '')}

def parse_args():
    args = {}
    if request.args:
        if 'status' in request.args:
            args['build_status'] = request.args.get('status')
        if 'device' in request.args:
            args['build_device'] = request.args.get('device')
        if 'version' in request.args:
            args['build_version'] = request.args.get('version')
        if 'type' in request.args:
            args['build_type'] = request.args.get('type')
        if 'date' in request.args:
            date = datetime.datetime.strptime(request.args.get('date'), '%Y-%m-%d').date()
            args['build_date'] = datetime.datetime.strptime(request.args.get('date'), '%Y-%m-%d').date()
    return args

@app.route('/')
def index():
    try:
        args = parse_args()
    except ValueError:
        return "Invalid Date", 400
    builds = models.Build.query.filter_by(**args).order_by(models.Build.build_date.desc(), models.Build.build_id).paginate(per_page=20)
    return render_template('builds.html', builds=builds)

@app.route('/runners/<string:runner>')
def runner(runner):
    try:
        args = parse_args()
    except ValueError:
        return "Invalid Date", 400
    runner = models.Runner.query.filter_by(runner_name=runner).first()
    args['build_runner'] = runner
    builds = models.Build.query.filter_by(**args).order_by(models.Build.build_date.desc(), models.Build.build_id).paginate(per_page=20)
    return render_template('runner.html', runner=runner, builds=builds)

@app.route("/runners/")
def runners():
    runners = models.Runner.query.order_by(models.Runner.runner_sponsor).all()
    return render_template('runners.html', runners=runners)

@app.route("/webhook", methods=('POST',))
def process_webhook():
    gitlab.webhooks.process(request)
    return "OK", 200

if __name__ == '__main__':
    app.run(host='0.0.0.0', port=5000)
